import sqlite3
import sys

import pymysql


def serialize(obj):
    return repr(obj)


def deserialize(txt):
    import ast
    return ast.literal_eval(txt)


def create_table_query(table_name=None, column_dict=None):
    '''
    :param table_name:
    :param column_dict:
    :return: query

    example dict

    column_dict = dict(
    id="int(11) PRIMARY KEY NOT NULL AUTO_INCREMENT",
    package_name="text",
    sku="text",
    price="float"
    )
    '''

    query = "CREATE TABLE IF NOT EXISTS"
    query = " ".join([query, table_name, '('])

    for k, v in column_dict.items():
        if k == list(column_dict.keys())[-1]:
            query = " ".join([query, k, v])
        else:
            query = " ".join([query, k, v, ','])

    # finally
    query = " ".join([query, ')'])

    return query


class CursorDict:
    '''
    Returns a dictionary after executing a db query instead of a cursor
    '''

    def __init__(self, db_name):
        '''
        Initialise object
        :param db_name: Database file to use
        '''
        self.__connection = sqlite3.connect(db_name)
        self.__cursor = self.__connection.cursor()

    def connection(self):
        return self.__connection

    def cursor(self):
        return self.__cursor

    def execute(self, query, args=None):
        '''
        :param query:
        :param args:
        :return:
        '''
        cursor = self.__cursor.execute(query)
        res = []
        rlist = list(cursor)
        keys = [str(r[0]) for r in cursor.description]
        for r in rlist:
            mdict = dict(zip(keys, r))
            res.append(mdict)
        return res

    def commit(self):
        self.connection.commit()

    def close(self):
        '''
        Close cursor
        :return:
        '''
        self.cursor.close()


class SqliteDataStore(object):
    def __init__(self, db_name=None, create_table_query=None):

        ''' Intialize the package, db connectionn and the cursor. '''
        try:
            import sqlite3
        except ImportError:
            print("sqlite3 package could not be imported. Exiting.")
            sys.exit(0)

        if not db_name:
            db_name = "temp_db.ten"

        self.cd = CursorDict(db_name)
        self.connection = self.cd.connection()
        self.cursor = self.cd.cursor()

        if create_table_query:
            self.cursor.execute(create_table_query)

    def run_query(self, query, parameters=[], dictionary_output=True):
        if dictionary_output:
            return self.cd.execute(query, parameters)

    def select_one(self, query, parameters=None):
        result_dict = None

        try:
            self.cursor.execute(query, parameters)
            result_dict = self.cursor.fetchone()
        except Exception as e:
            print(e)
            # connection.close()
        return result_dict

    def select_all(self, query):
        try:
            self.cursor.execute(query)
            return self.cursor.fetchall()

        except Exception as e:
            print(e)
            # connection.close()

    def insert(self, query, parameters=None):
        try:
            self.cursor.execute(query, parameters)
            self.connection.commit()
            return True
        except Exception as e:
            print("insert error", e)
            return None

    def delete(self, query, parameters=None):
        try:
            self.cursor.execute(query, parameters)
            self.connection.commit()
            return True
        except Exception as e:
            print("insert error", e)

    def select_all_ids(self, table):
        try:
            self.cursor.execute("select ids from %s" % table)
            r = self.cursor.fetchall()
            return [i[0] for i in r]
        except Exception as e:
            print(e)


class MySqlDataStore(object):
    def __init__(self, host=None, user=None, passwd=None, db=None):
        self.connection = pymysql.connect(host=host, user=user, passwd=passwd, db=db,
                                          cursorclass=pymysql.cursors.DictCursor)

        self.cursor = self.connection.cursor()

    def create_table(self, query):
        self.cursor.execute(query)

    def get_connection(self):
        return self.connection

    def select_one(self, query, parameters=None):
        result_dict = None

        try:
            with self.connection.cursor() as cursor:
                cursor.execute(query, parameters)
                result_dict = cursor.fetchone()
        except Exception as e:
            print(e)
            # connection.close()
        return result_dict

    def select_all(self, query, parameters=None):
        result_dict = None

        try:
            with self.connection.cursor() as cursor:
                cursor.execute(query, parameters)
                result_dict = cursor.fetchall()
        except Exception as e:
            print(e)
        return result_dict

    def insert(self, query, parameters=None):
        try:
            with self.connection.cursor() as cursor:
                # [self.jid,self.gid,self.message,self.timestamp,self.is_group]
                # "insert into messages (jid,gid,message,timestamp,isgroup) values (%s,%s,%s,%s,%s)"
                cursor.execute(query, parameters)
                self.connection.commit()
                return True

        except Exception as e:
            print("insert error", e)
            # connection.close()
            return False

    def insert_many(self, query, parameters=None):

        try:
            with self.connection.cursor() as cursor:
                # [self.jid,self.gid,self.message,self.timestamp,self.is_group]
                cursor.executemany(query, parameters)
                self.connection.commit()
                return True
        except Exception as e:
            print("insert error", e)
            # connection.close()
        return False

    def update(self, query, parameters=None):

        try:
            with self.connection.cursor() as cursor:

                result = cursor.execute(query, parameters)
                if result == 0:
                    return "update failed"

                if result == 1:
                    self.connection.commit()
                    return "update successful"

        except Exception as e:
            print("insert error", e)
            # connection.close()
            return "execution error"
